# $Id: Config.pm,v 1.1 2004/05/15 13:48:24 rich Exp $

package Subscripto::Config;

use File::stat;
use Fcntl ':mode';
use Carp;
use IO::File;
use XML::LibXML;

sub new
{
    my $class = shift(@_);
    my $dir = shift(@_);
    my $self = bless({}, $class);

    if (!defined($dir))
    {
	$dir = $ENV{HOME}.'/.subscripto';
    }

    # Check the permissions on $dir. If they're not 0700, barf.
    if (-d $dir)
    {
	my $sb = stat($dir);
	if ($sb->mode & (S_IRWXG|S_IRWXO))
	{
	    carp
		'Refusing to start - unsafe permissions on '
		.$dir.' - use 0700';
	}
    }

    # Set up defaults.
    $self->{dir} = $dir;
    $self->{file} = $dir.'/config.xml';

    $self->_clear();

    # If the file exists and this fails, abort. Otherwise, ignore it.
    if (!$self->load() && -r $self->{file})
    {
	carp 'Unable to parse configuration file "'.$file.'"';
    }

    return $self;
}

sub _clear
{
    my $self = shift(@_);

    $self->{xml} = undef;
    $self->{xmlparser} = undef;

    $self->{folders} = [];
}

sub load
{
    my $self = shift(@_);
    my $fh = new IO::File('<'.$self->{file});
    my $ret = 0; # Fail by default.

    if (defined($fh))
    {
	$self->{xmlparser} = XML::LibXML->new();
	if (defined($self->{xmlparser}))
	{
	    $self->{xml} = $self->{xmlparser}->parse_fh($fh);
	}
	$fh->close();
    }

    return $self->_parse();
}

sub _parse
{
    my $self = shift(@_);
    my $ret = 1; # Succeed by default.

    if (defined($self->{xml}))
    {
	# Parse the config.
	my $match = $self->{xml}->findnodes('/subscripto/sources/mailbox');
	my $ctx;
	my $node;

	foreach $node ($match->get_nodelist())
	{
	    push(@{$self->{folders}}, $node->textContent());
	}
    }
    else
    {
	$ret = 0;
    }

    return $ret;
}

sub reload
{
    my $self = shift(@_);

    $self->_clear();
    return $self->load();
}

sub save
{
    my $self = shift(@_);

    -d $self->{dir}
        || mkdir($self->{dir}, 0700)
	|| die 'Unable to create '.$self->{dir};

    # TODO: Write the configuration.
}

sub get_folders
{
    my $self = shift(@_);

    return @{$self->{folders}};
}

1;
